
/* ----------------------------------------------------------------
 * gregorian calendar algorithms
 * Fliegel and Van Flandern's algorithms,
 *	    _Explanatory Supplement_ (1992), p. 604;
 * source = CACM 11 (1968) 657
 *	(CACM = Communications of the Association for Computing Machinery)
 * the algorithm as printed in the _Explanatory Supplement_
 *
 * use jd.h
 *
 * GregorianCalendarToJD( year, month, day ) returns jd for noon of the date
 * ----------------------------------------------------------------
 */

long
GregorianCalendarToJD( year, month, day )
int year;
int month;
int day;
{
  long jd;
  long fmonth, fyear;

  fmonth = month;
  fyear = year; 

  jd = (1461 * (fyear + 4800 + (fmonth-14)/12))/4;
  jd +=  (367*(fmonth - 2 - 12*((fmonth-14)/12)))/12; 
  jd -= (3*((fyear + 4900 + (fmonth-14)/12)/100))/4;
  jd += day - 32075.; 
    
  return jd;

}



void
xJDtoGregorianCalendar( jdate, year, month, day )
  double jdate;
  int *year;
  int *month;
  int *day;
{
  long jd;
    
  jd = jdate + 0.5;
  JDtoGregorianCalendar( jd, year, month, day ); 

}

void
JDtoGregorianCalendar( jd, year, month, day )
  long jd;
  int *year;
  int *month;
  int *day;
{
  long i, j, el, n;
    
  el = jd + 68569;
  n = (4*el)/146097;
  el = el - (146097*n + 3)/4;
  i = (4000 * (el+1)) / 1461001;
  el = el - (1461*i) / 4 + 31;
  j = (80*el) / 2447;
  
  *day = el - (2447*j)/80;

  el = j/11;

  *month = j + 2 - 12*el;
  *year = 100 * (n-49) + i + el;

}



/* ----------------------------------------------------------------
 * conversions for JULIAN CALENDAR DATE . . . !!   
 * use jd.h
 * the source in _Explanatory Supplement_ (1992), p. 606,
 * was not previously published. 
 * ----------------------------------------------------------------
 */

long
JulianCalendarToJD( year, month, day ) 
  int year;
  int month;
  int day;
{
  long jd;
  long lyear, lmonth, lday;

  /* loss of precision will kill us if we don't convert to long: */ 
  lyear = year;
  lmonth = month;
  lday = day; 
   

  jd = 367 * lyear
    - ( 7 * (lyear + 5001 +  (lmonth-9)/7 )  ) / 4 
    + (275 * lmonth)/9 + lday + 1729777;

  return jd;

}


void
xJDtoJulianCalendar( jdate, year, month, day )
  double jdate;
  int *year;
  int *month;
  int *day;
{
  long jd;
  long i, j, el, n;
    
  jd = jdate + 0.5;
  JDtoJulianCalendar( jd, year, month, day ); 

}


void 
JDtoJulianCalendar( jd, year, month, day )
  long jd; 
  int *year;
  int *month;
  int *day;
{
  long i, j, k, el, n;
    
  j = jd + 1402;
  k = (j-1)/1461;
  el = j - 1461*k;
  n = (el-1)/365 - el/1461;
  i = el - 365*n + 30;
  j = (80*i) / 2447;
  *day = i - (2447 *j) / 80;
  i = j/11;
  *month = j + 2 - 12*i;
  *year = 4*k + n + i - 4716;

}


/* ----------------------------------------------------------------
 * setCalendarTransition sets the global variable 'jdtransition'
 *    jdtransition   is the  first julian day number of the change to the gregorian calendar
 *    jdtransition-1 is the  last  julian day number of julian calendar 
 *    in different places, the transition is at quite different dates
 * 	Britain and dependencies: 
 *		1752/09/03 julian = 1752/09/14 gregorian = jd 2361222
 *	Italy, Spain
 *		1582/10/05 julian = 1582/10/15 gregorian = jd 2299161
 *	USSR
 *		1918/02/01 julian = 1918/02/14 gregorian 
 *	France, Germany -- various dates in various places.
 *		Catholic parts tend to be in 1583, Protestant in 1700's.  
 *
 * inputs
 *    the environmental variable 'JDTRANS',
 *	  which is the first julian day number of the shift to gregorian calendar
 * outputs
 *    none 
 * side effects
 *    sets the global variable 'jdtransition'
 */

void setCalendarTransition()
{
  char *s;

  if (  (s=getenv("JDTRANS")) != NULL)
    jdtransition = atol( s ); 
}




/* ----------------------------------------------------------------
 * setGregorian sets the global variable 'gregorian' 
 *    gregorian==1 --> convert julian day numbers to/from gregorian calendar dates
 *	  (actually, any gregorian != 0 will do) 
 *    gregorian==0 --> convert julian day numbers to/from julian    calendar dates
 * inputs
 *    the julian day number of the date for which 'gregorian' is to be set 
 *    the global variable 'jdtransition',
 *	  which is the first julian day number of the shift to gregorian calendar
 * outputs
 *    none
 * side effects
 *    sets the global variable 'gregorian'
 */

void
setGregorian( jd )
long jd;
{
  if (jd >= jdtransition)
    gregorian = 1;
  else
    gregorian = 0;

}



/* ----------------------------------------------------------------
 * jdnoon converts from a double-float julian date to a long-int julian date,
 * and converts to the julian date of noon
 * of the day in which the double-float JD occurs
 * ----------------------------------------------------------------
 */


long
jdnoon( jdx )
  double jdx;
{
  long jd;

  jd = jdx + 0.5;

  return jd;

}


/* ----------------------------------------------------------------
 * selective routines, directed according to global variable 'gregorian'
 * ----------------------------------------------------------------
 */ 

void
JDtoCalendar( jd, year, month, day )
  long jd; 
  int *year;
  int *month;
  int *day;
{
  if (gregorian) JDtoGregorianCalendar( jd, year, month, day );
  else           JDtoJulianCalendar(    jd, year, month, day );
}


void
xJDtoCalendar( jdate, year, month, day )
  double jdate;
  int *year;
  int *month;
  int *day;
{
  if (gregorian) xJDtoGregorianCalendar( jdate, year, month, day );
  else           xJDtoJulianCalendar( jdate, year, month, day );
}


/* ----------------------------------------------------------------
 * this one is tricky -- you assume you are on the gregorian calendar,
 *    and if that doesn't work, switch to julian.
 *    but you have to test to see whether it works . . .
 * inputs
 *    year, month, day for which the question to be answered is <which calendar?>
 * outputs
 *    none
 * side effects
 *    if the date is post-transition, set the global variable gregorian = 1
 *    if the date is  pre-transition, set the global variable gregorian = 0
 *    if the date is in the in-between region, not a valid date, complain and crash
 * assumptions
 *    that the jdtransition,
 *    the julian day number
 *        of the transition from julian to gregorian calendars,
 *    is set as you would like it to be 
 * ----------------------------------------------------------------
 */


void
dateSetGregorian( year, month, day )
int year;
int month;
int day;
{
  long jdtestGregorian;
  long jdtestJulian; 

  jdtestGregorian = GregorianCalendarToJD( year, month, day );
  jdtestJulian    =    JulianCalendarToJD( year, month, day );


  if (jdtestGregorian >= jdtransition)
  {
    gregorian = 1; 
    return; 
  }
  if (jdtestJulian    <  jdtransition)
  {
    gregorian = 0; 
    return;
  }
  
  /* if you got here, the date is in the not-allowed region;
   * print a complaint
   * if you WANT pre-transition Gregorian or post-transition Julian conversions,
   * you have to ask for them explicitly.
   * so we crash! 
   */

  printf(" your date is in the not-allowed region: %d %d %d \n", year, month, day );
  printf(" last     julian calendar date = %s\n",    xJDtoJulianString( (double) (jdtransition-1) ) );
  printf(" first gregorian calendar date = %s\n", xJDtoGregorianString( (double) (jdtransition)   ) );

  exit(0); 
  
}

  

long
CalendarToJD( year, month, day )
int year;
int month;
int day;
{
  dateSetGregorian( year, month, day );

  if (gregorian) return GregorianCalendarToJD( year, month, day );
  else           return    JulianCalendarToJD( year, month, day );

}

